@@ -27,15 +27,6 @@ def cache_to_django(
         django_model,
         cache_conf,
 ):
-    if redis_root.performance['enabled']:
-        if 'cache_to_django' not in redis_root.performance.keys():
-            redis_root.performance['cache_to_django'] = {
-                'requests': 0,
-                'time': 0
-            }
-        redis_root.performance['cache_to_django']['requests'] += 1
-        started_in = datetime.datetime.now()
-
     async def run_cache_to_django(
             redis_root,
             django_model,
@@ -101,9 +92,6 @@ def cache_to_django(
         django_model,
         cache_conf,
     ))
-    if redis_root.performance['enabled']:
-        ended_in = datetime.datetime.now()
-        redis_root.performance['cache_to_django']['time'] += (ended_in - started_in).total_seconds()
 
 
 async def update_or_create_django_instance_from_redis_instance(
@@ -311,15 +299,6 @@ def django_to_cache(
         django_model,
         cache_conf,
 ):
-    if redis_root.performance['enabled']:
-        if 'django_to_cache' not in redis_root.performance.keys():
-            redis_root.performance['django_to_cache'] = {
-                'requests': 0,
-                'time': 0
-            }
-        redis_root.performance['django_to_cache']['requests'] += 1
-        started_in = datetime.datetime.now()
-
     async def run_django_to_cache(
             redis_root,
             django_model,
@@ -334,22 +313,25 @@ def django_to_cache(
         async_tasks = []
         completed = 0
         to_complete = len(django_instances)
+        if django_instances:
+            all_redis_instances = redis_root.get(django_model)
 
-        for django_instance in django_instances:
-            async_tasks.append(
-                update_or_create_redis_instance_from_django_instance(
-                    redis_root,
-                    django_instance,
-                    cache_conf,
+            for django_instance in django_instances:
+                async_tasks.append(
+                    update_or_create_redis_instance_from_django_instance(
+                        redis_root,
+                        django_instance,
+                        all_redis_instances,
+                        cache_conf,
+                    )
                 )
-            )
-            completed += 1
-            if len(async_tasks) == async_chunk_size or completed == to_complete:
-                await asyncio.gather(*async_tasks)
-                async_tasks = []
-                print(
-                    f'{datetime.datetime.now()} - '
-                    f'Written {completed} {django_model} instances from django to cache')
+                completed += 1
+                if len(async_tasks) == async_chunk_size or completed == to_complete:
+                    await asyncio.gather(*async_tasks)
+                    async_tasks = []
+                    print(
+                        f'{datetime.datetime.now()} - '
+                        f'Written {completed} {django_model} instances from django to cache')
 
         if cache_conf['delete']:
             async def check_if_need_to_delete_redis_instance(redis_root, django_instances_ids, redis_instance):
@@ -377,27 +359,20 @@ def django_to_cache(
             cache_conf,
         )
     )
-    if redis_root.performance['enabled']:
-        ended_in = datetime.datetime.now()
-        redis_root.performance['django_to_cache']['time'] += (ended_in - started_in).total_seconds()
 
 
 async def update_or_create_redis_instance_from_django_instance(
         redis_root,
         django_instance,
+        all_redis_instances,
         cache_conf,
 ):
-    if redis_root.performance['enabled']:
-        if 'update_or_create_redis_instance_from_django_instance' not in redis_root.performance.keys():
-            redis_root.performance['update_or_create_redis_instance_from_django_instance'] = {
-                'requests': 0,
-                'time': 0
-            }
-        redis_root.performance['update_or_create_redis_instance_from_django_instance']['requests'] += 1
-        started_in = datetime.datetime.now()
 
     django_instance_model = django_instance.__class__
-    old_redis_instance = redis_root.get(django_instance_model, django_id=django_instance.id)
+    old_redis_instance = get_redis_instance_by_django_id_from_all_redis_instances(
+        django_instance.id,
+        all_redis_instances
+    )
     redis_dict = await django_instance_to_redis_params(
         redis_root,
         django_instance,
@@ -413,20 +388,19 @@ async def update_or_create_redis_instance_from_django_instance(
         )
         result_redis_instance_id = redis_instance['id']
     else:
-        old_redis_instance = old_redis_instance[0]
         fields_to_update = check_fields_need_to_update(
             old_redis_instance,
             redis_dict,
         )
-        redis_root.update(django_instance_model, old_redis_instance,
-                          **fields_to_update
-                          )
-        redis_instance = redis_root.get(django_instance_model, django_id=django_instance.id)[0]
-        result_redis_instance_id = redis_instance['id']
+        if fields_to_update:
+            redis_root.update(django_instance_model, old_redis_instance,
+                              **fields_to_update
+                              )
+            redis_instance = redis_root.get(django_instance_model, django_id=django_instance.id)[0]
+            result_redis_instance_id = redis_instance['id']
+        else:
+            result_redis_instance_id = old_redis_instance['id']
 
-    if redis_root.performance['enabled']:
-        ended_in = datetime.datetime.now()
-        redis_root.performance['update_or_create_redis_instance_from_django_instance']['time'] += (ended_in - started_in).total_seconds()
 
     return result_redis_instance_id
 
@@ -451,14 +425,6 @@ async def django_instance_field_to_redis_value(
         django_field,
         cache_conf,
 ):
-    if redis_root.performance['enabled']:
-        if 'django_instance_field_to_redis_value' not in redis_root.performance.keys():
-            redis_root.performance['django_instance_field_to_redis_value'] = {
-                'requests': 0,
-                'time': 0
-            }
-        redis_root.performance['django_instance_field_to_redis_value']['requests'] += 1
-        started_in = datetime.datetime.now()
 
 
     save_related_models = cache_conf['save_related_models']
@@ -470,25 +436,37 @@ async def django_instance_field_to_redis_value(
         else:
             django_foreign_key_instance = django_field_value
             if save_related_models:
-                redis_foreign_key_instance = await get_or_create_redis_instance_from_django_instance(
-                    django_foreign_key_instance,
-                    redis_root,
-                    cache_conf,
-                )
-                redis_value = redis_foreign_key_instance
+                if django_foreign_key_instance:
+                    django_instance_model = django_foreign_key_instance.__class__
+                    all_redis_instances = redis_root.get(django_instance_model)
+                    redis_foreign_key_instance = await get_or_create_redis_instance_from_django_instance(
+                        django_foreign_key_instance,
+                        redis_root,
+                        all_redis_instances,
+                        cache_conf,
+                    )
+                    redis_value = redis_foreign_key_instance
+                else:
+                    redis_value = None
             else:
                 redis_value = django_foreign_key_instance.id
     elif django_field.__class__ == django_models.ManyToManyField:
         django_many_to_many_instances = await django_sync_to_async_list(django_field_value.all)
         if save_related_models:
-            redis_value = [
-                await get_or_create_redis_instance_from_django_instance(
-                    django_many_to_many_instance,
-                    redis_root,
-                    cache_conf,
-                )
-                for django_many_to_many_instance in django_many_to_many_instances
-            ]
+            if len(django_many_to_many_instances):
+                django_instance_model = django_many_to_many_instances[0].__class__
+                all_redis_instances = redis_root.get(django_instance_model)
+                redis_value = [
+                    await get_or_create_redis_instance_from_django_instance(
+                        django_many_to_many_instance,
+                        redis_root,
+                        all_redis_instances,
+                        cache_conf,
+                    )
+                    for django_many_to_many_instance in django_many_to_many_instances
+                ]
+            else:
+                redis_value = []
         else:
             redis_value = [
                 django_many_to_many_instance.id
@@ -501,9 +479,6 @@ async def django_instance_field_to_redis_value(
             redis_value = None
 
 
-    if redis_root.performance['enabled']:
-        ended_in = datetime.datetime.now()
-        redis_root.performance['django_instance_field_to_redis_value']['time'] += (ended_in - started_in).total_seconds()
 
     return redis_value
 
@@ -513,18 +488,11 @@ async def django_instance_to_redis_params(
         django_instance,
         cache_conf,
 ):
-    if redis_root.performance['enabled']:
-        if 'django_instance_to_redis_params' not in redis_root.performance.keys():
-            redis_root.performance['django_instance_to_redis_params'] = {
-                'requests': 0,
-                'time': 0
-            }
-        redis_root.performance['django_instance_to_redis_params']['requests'] += 1
-        started_in = datetime.datetime.now()
 
     redis_params = {}
     exclude_fields = cache_conf['exclude_fields']
-    for i, django_field in enumerate(django_instance.__class__._meta.get_fields()):
+    django_fields = django_instance.__class__._meta.get_fields()
+    for i, django_field in enumerate(django_fields):
         if not django_field.__class__.__name__.endswith('Rel'):
             django_field_name = django_field.name
             allowed = True
@@ -541,20 +509,42 @@ async def django_instance_to_redis_params(
                     if redis_param:
                         redis_params[django_field_name] = redis_param
 
-    if redis_root.performance['enabled']:
-        ended_in = datetime.datetime.now()
-        redis_root.performance['django_instance_to_redis_params']['time'] += (ended_in - started_in).total_seconds()
 
     return redis_params
 
 
+def get_redis_instance_by_django_id_from_all_redis_instances(
+        django_id,
+        all_redis_instances
+):
+    all_django_ids = []
+    for redis_instance in all_redis_instances:
+        django_id = None
+        if 'django_id' in redis_instance.keys():
+            django_id = redis_instance['django_id']
+        all_django_ids.append(django_id)
+    try:
+        index_of_redis_instance = all_django_ids.index(django_id)
+        redis_instance = all_redis_instances[index_of_redis_instance]
+    except:
+        redis_instance = None
+    return redis_instance
+
+
 async def get_or_create_redis_instance_from_django_instance(
         django_instance,
         redis_root,
+        all_redis_instances,
         cache_conf,
 ):
+
+    redis_instance = get_redis_instance_by_django_id_from_all_redis_instances(
+        django_instance.id,
+        all_redis_instances
+    )
+    print(django_instance_model, django_instance.id)
+
     django_instance_model = django_instance.__class__
-    redis_instance = redis_root.get(django_instance_model, django_id=django_instance.id)
     if not redis_instance:
         redis_dict = await django_instance_to_redis_params(
             redis_root,
@@ -565,8 +555,9 @@ async def get_or_create_redis_instance_from_django_instance(
             django_model=django_instance_model,
             django_id=django_instance.id,
             **redis_dict
-        ).save()
+        )
     else:
         redis_instance = redis_instance[0]
 
+
     return redis_instance
