@@ -130,12 +130,18 @@ class RedisDateTime(RedisString):
 class RedisForeignKey(RedisNumber):
 
     def __init__(self, model=None, save_related_models=False, *args, **kwargs):
-        if not issubclass(model, RedisModel):
+        allowed = True
+        if model is None:
+            allowed = False
+        elif not issubclass(model, RedisModel):
+            allowed = False
+        if not allowed:
             raise Exception(f'{model.__name__} class is not RedisModel')
-        self.model = model
-        check_types(save_related_models, bool)
-        self.save_related_models = save_related_models
-        super().__init__(*args, **kwargs)
+        else:
+            self.model = model
+            check_types(save_related_models, bool)
+            self.save_related_models = save_related_models
+            super().__init__(*args, **kwargs)
 
     def _get_id_from_instance_dict(self):
         if self.value:
@@ -143,11 +149,9 @@ class RedisForeignKey(RedisNumber):
                 if 'id' in self.value.keys():
                     self.value = self.value['id']
                 else:
-                    raise Exception(
-                        f"{self.value} has no key 'id', please provide serialized instance or dict like " + "{'id': 1, ...}")
+                    raise Exception(f"{self.value} has no key 'id', please provide serialized instance or dict like " + "{'id': 1, ...}")
             else:
-                raise Exception(
-                    f'{self.value} type is not dict, please provide serialized instance or dict like ' + "{'id': 1, ...}")
+                raise Exception(f'{self.value} type is not dict, please provide serialized instance or dict like ' + "{'id': 1, ...}")
         return self.value
 
     def clean(self):
@@ -156,29 +160,15 @@ class RedisForeignKey(RedisNumber):
             self.value = get_ids_from_untyped_data(self.value)[0]
         return super().clean()
 
-    def _get_instance_by_id(self, id, redis_root):
-        model_name = self.model.__name__
-        instance = {
-            'id': id
-        }
-        fast_get_results = redis_root.fast_get_keys_values(f'{redis_root.prefix}:{model_name}:{id}:*')
-        for key, raw_value in fast_get_results.items():
-            prefix, model_name, instance_id, field_name = key.split(':')
-            value = redis_root.deserialize_value(raw_value, model_name, field_name)
-            instance[field_name] = value
-        return instance
-
     def deserialize_value(self, value, redis_root):
         self.deserialize_value_check_null(value, redis_root)
         if value != 'null':
-            value = super().deserialize_value(value, redis_root)
             check_types(value, int)
-            if self.save_related_models:
-                value = self._get_instance_by_id(value, redis_root)
-            else:
-                value = {
-                    'django_id': value
-                }
+            model_name = self.model.__name__
+            instance_id = value
+            value = redis_root._get_instances_by_key(f'{redis_root.prefix}:{model_name}:{instance_id}')[instance_id]
+        else:
+            value = None
         return value
 
 
@@ -240,12 +230,18 @@ class RedisList(RedisJson):
 class RedisManyToMany(RedisList):
 
     def __init__(self, model=None, save_related_models=False, *args, **kwargs):
-        if not issubclass(model, RedisModel):
+        allowed = True
+        if model is None:
+            allowed = False
+        elif not issubclass(model, RedisModel):
+            allowed = False
+        if not allowed:
             raise Exception(f'{model.__name__} class is not RedisModel')
-        self.model = model
-        check_types(save_related_models, bool)
-        self.save_related_models = save_related_models
-        super().__init__(*args, **kwargs)
+        else:
+            self.model = model
+            check_types(save_related_models, bool)
+            self.save_related_models = save_related_models
+            super().__init__(*args, **kwargs)
 
     def clean(self):
         self.value = self.check_value()
@@ -253,34 +249,20 @@ class RedisManyToMany(RedisList):
             self.value = get_ids_from_untyped_data(self.value)
         return super().clean()
 
-    def _get_instance_by_id(self, id, redis_root):
-        model_name = self.model.__name__
-        instance = {
-            'id': id
-        }
-        fast_get_results = redis_root.fast_get_keys_values(f'{redis_root.prefix}:{model_name}:{id}:*')
-        for key, raw_value in fast_get_results.items():
-            prefix, model_name, instance_id, field_name = key.split(':')
-            value = redis_root.deserialize_value(raw_value, model_name, field_name)
-            instance[field_name] = value
-        return instance
-
     def deserialize_value(self, value, redis_root):
         self.deserialize_value_check_null(value, redis_root)
         if value != 'null':
             value = super().deserialize_value(value, redis_root)
-            check_types(value, list)
-            if self.save_related_models:
-                values_list = [
-                    self._get_instance_by_id(redis_model_instance_dict, redis_root)
-                    for redis_model_instance_dict in value
-                ]
-            else:
-                values_list = [
-                    {'django_id': django_id}
-                    for django_id in value
-                ]
-            value = values_list
+            model_name = self.model.__name__
+            isntances_ids = value
+            instances_list = []
+            for instance_id in isntances_ids:
+                instances = redis_root._get_instances_by_key(f'{redis_root.prefix}:{model_name}:{instance_id}')
+                if len(instances.keys()):
+                    instance = instances[instance_id]
+                    instances_list.append(instance)
+            value = instances_list
+
         return value
 
 
@@ -305,19 +287,20 @@ class RedisBool(RedisNumber):
         return value
 
 
-class RedisDecimal(RedisNumber):
+class RedisDecimal(RedisString):
 
     def clean(self):
         self.value = self.check_value()
         if self.value not in [None, 'null']:
             check_types(self.value, (int, float, decimal.Decimal))
-            self.value = float(self.value)
         return super().clean()
 
     def deserialize_value(self, value, redis_root):
         self.deserialize_value_check_null(value, redis_root)
         if value != 'null':
             value = decimal.Decimal(value)
+        else:
+            value = None
         return value
 
 
@@ -749,6 +732,20 @@ class RedisRoot:
                     }
         return checked_instances
 
+    def _get_instances_by_key(self, key):
+        raw_instances = self.fast_get_keys_values(key)
+        instances = {}
+        for instance_key, fields_json in raw_instances.items():
+            prefix, model_name, instance_id = instance_key.split(':')
+            instance_id = int(instance_id)
+            fields_dict = json.loads(fields_json)
+            for field_name, raw_value in fields_dict.items():
+                value = self.deserialize_value(raw_value, model_name, field_name)
+                if instance_id not in instances.keys():
+                    instances[instance_id] = {}
+                instances[instance_id][field_name] = value
+        return instances
+
     def _filter_value(self, value, filter_type, filter_by):
         allowed = True
         if filter_type == 'exact':
@@ -849,11 +846,14 @@ class RedisRoot:
         instances = self._get_instances_from_instances_with_allowed(instances_with_allowed)
         return instances
 
+    def get_by_redis_model(self, redis_model, return_dict=False, **filters):
+        instances = self._get_all_redis_model_instances(redis_model, filters)
+        return self._return_with_format(instances, return_dict)
 
     def get(self, django_model, return_dict=False, **filters):
         redis_model = self.get_or_create_redis_model_from_django_model(django_model)
-        instances = self._get_all_model_instances(redis_model, filters)
-        return self._return_with_format(instances, return_dict)
+        result = get_by_redis_model(redis_model, return_dict=False, **filters)
+        return result
 
     def order(self, instances, field_name):
         reverse = False
@@ -1074,7 +1074,7 @@ class RedisModel:
 
     def _get_new_id(self):
         redis_root = self.__model_data__['redis_root']
-        instances_with_ids = redis_root.get(self.__class__, return_dict=True)
+        instances_with_ids = redis_root.get_by_redis_model(self.__class__, return_dict=True)
         all_ids = [int(instance_id) for instance_id in list(instances_with_ids.keys())]
         if all_ids:
             max_id = max(all_ids)
